# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.


"""Tests the album view of the gallery app"""

from testtools.matchers import Equals, GreaterThan, LessThan
from autopilot.matchers import Eventually

from lomiri_gallery_app.emulators.album_view import AlbumView
from lomiri_gallery_app.emulators.albums_view import AlbumsView
from lomiri_gallery_app.emulators.media_selector import MediaSelector
from lomiri_gallery_app.emulators.photo_viewer import PhotoViewer
from lomiri_gallery_app.emulators.photo_viewer import PopupPhotoViewer
from lomiri_gallery_app.emulators import album_editor
from lomiri_gallery_app.tests import GalleryTestCase

import os
from time import sleep
from unittest import skip


class TestAlbumView(GalleryTestCase):
    """Tests the album view of the gallery app"""

    @property
    def album_view(self):
        return AlbumView(self.app)

    @property
    def albums_view(self):
        return AlbumsView(self.app)

    @property
    def media_selector(self):
        return MediaSelector(self.app)

    @property
    def photo_viewer(self):
        return PhotoViewer(self.app)

    def setUp(self):
        self.ARGS = []
        super(TestAlbumView, self).setUp()
        self.switch_to_albums_tab()

    def test_album_view_open_photo(self):
        self.open_first_album()
        photo = self.album_view.get_first_photo()
        self.click_item(photo)
        sleep(5)
        photo_view = self.main_view.wait_select_single("PopupPhotoViewer")
        self.assertThat(photo_view.visible, Eventually(Equals(True)))

    def test_album_view_flipping(self):
        # For some reason here the album at position 0 in the autopilot list is
        # actually the second album, they seem to be returned in reverse order.
        self.open_album_at(1)

        spread = self.album_view.get_spread_view()

        # check that we can page to the cover and back (we check for lesser
        # than 1 because it can either be 0 if we are on a one page spread
        # or -1 if we are on a two page spread, for example on desktop)
        self.album_view.swipe_page_left(1)
        self.assertThat(spread.viewingPage, Eventually(LessThan(1)))
        self.album_view.swipe_page_right(0)
        self.assertThat(spread.viewingPage, Eventually(Equals(1)))

        # drag to next page and check we have flipped away from page 1
        # can't check precisely for page 2 because depending on form factor
        # and orientation we might be displaying two pages at the same time
        self.album_view.swipe_page_right(1)
        self.assertThat(spread.viewingPage, Eventually(GreaterThan(1)))

    def test_add_photo(self):
        self.open_first_album()
        num_photos_start = self.album_view.number_of_photos()
        self.assertThat(num_photos_start, Equals(1))

        # should click away of any photo to toggle header
        photo = self.album_view.get_first_photo()
        x, y, w, h = photo.globalRect
        self.pointing_device.move(x + 40, y + h + 40)
        self.pointing_device.click()

        # open media selector but cancel
        self.album_view.click_action_button("addButton")
        self.media_selector.ensure_fully_open()

        self.main_view.get_header().click_custom_back_button()
        sleep(1)

        num_photos = self.album_view.number_of_photos()
        self.assertThat(num_photos, Equals(num_photos_start))

        # open media selector and add a photo
        self.album_view.click_action_button("addButton")
        self.media_selector.ensure_fully_open()

        photo = self.media_selector.get_second_photo()
        checkbox = photo.select_single(objectName="selectionCheckbox")
        self.click_item(checkbox)
        self.main_view.get_header().click_action_button("addButton")

        self.assertThat(
            lambda: self.album_view.number_of_photos(),
            Eventually(Equals(num_photos_start + 1)))

    def test_remove_photo_from_album(self):
        self.open_first_album()
        num_photos_start = self.album_view.number_of_photos()
        self.assertThat(num_photos_start, Equals(1))

        path = self.album_view.click_first_photo()

        self.assertThat(lambda: os.path.exists(path),
                        Eventually(Equals(True)))

        photo_view = self.album_view.get_album_photo_view()
        self.assertThat(photo_view.visible, Eventually(Equals(True)))

        photo_viewer_popup = self.main_view.wait_select_single(
            PopupPhotoViewer, visible=True)
        photo_viewer_popup.click_action_button("deleteButton")
        self.album_view.click_remove_from_album_remove_button()

        self.assertThat(lambda: self.album_view.number_of_photos(),
                        Eventually(Equals(num_photos_start - 1)))

        self.assertThat(lambda: os.path.exists(path),
                        Eventually(Equals(True)))

    def test_remove_photo_from_album_and_delete(self):
        self.open_first_album()
        num_photos_start = self.album_view.number_of_photos()
        self.assertThat(num_photos_start, Equals(1))

        path = self.album_view.click_first_photo()

        self.assertThat(lambda: os.path.exists(path),
                        Eventually(Equals(True)))

        photo_view = self.album_view.get_album_photo_view()
        self.assertThat(photo_view.visible, Eventually(Equals(True)))

        photo_viewer_popup = self.main_view.wait_select_single(
            PopupPhotoViewer, visible=True)
        photo_viewer_popup.click_action_button("deleteButton")
        self.album_view.click_remove_from_album_delete_button()

        self.assertThat(lambda: self.album_view.number_of_photos(),
                        Eventually(Equals(num_photos_start - 1)))

        self.assertThat(lambda: os.path.exists(path),
                        Eventually(Equals(False)))

    def test_cancel_remove_photo_from_album(self):
        self.open_first_album()
        num_photos_start = self.album_view.number_of_photos()
        self.assertThat(num_photos_start, Equals(1))

        path = self.album_view.click_first_photo()

        self.assertThat(lambda: os.path.exists(path),
                        Eventually(Equals(True)))

        photo_view = self.album_view.get_album_photo_view()
        self.assertThat(photo_view.visible, Eventually(Equals(True)))

        photo_viewer_popup = self.main_view.wait_select_single(
            PopupPhotoViewer, visible=True)
        photo_viewer_popup.click_action_button("deleteButton")

        self.album_view.click_remove_from_album_cancel_button()

        photo_viewer_popup.click_action_button("backButton")

        self.assertThat(lambda: self.album_view.number_of_photos(),
                        Eventually(Equals(num_photos_start)))

        self.assertThat(lambda: os.path.exists(path),
                        Eventually(Equals(True)))

    def test_add_photo_to_new_album(self):
        albums = self.albums_view.number_of_albums_in_albums_view()
        self.main_view.get_header().click_action_button("addButton")
        self.ui_update()

        editor = self.app.select_single(album_editor.AlbumEditor)
        editor.ensure_fully_open()
        editor.add_photos()
        self.media_selector.ensure_fully_open()

        photo = self.media_selector.get_second_photo()
        checkbox = photo.select_single(objectName="selectionCheckbox")
        self.click_item(checkbox)
        self.main_view.get_header().click_action_button("addButton")
        self.ui_update()
        self.assertThat(
            lambda: self.albums_view.number_of_albums_in_albums_view(),
            Eventually(Equals(albums+1))
        )

        self.open_first_album()
        self.assertThat(
            lambda: self.album_view.number_of_photos(),
            Eventually(Equals(1)))

    @skip("Temporarily disable as it fails in some cases, supposedly due to "
          "problems with the infrastructure")
    def test_save_state(self):
        self.main_view.close_toolbar()
        self.open_first_album()

        id = self.album_view.get_album_view().albumId

        self.ensure_app_has_quit()
        self.start_app()

        view = self.album_view.get_album_view()
        self.assertThat(view.visible, Eventually(Equals(True)))
        self.assertThat(view.albumId, Eventually(Equals(id)))

    @skip("Temporarily disable as it fails in some cases, supposedly due to "
          "problems with the infrastructure")
    def test_no_save_state_on_back(self):
        self.main_view.close_toolbar()
        self.open_first_album()
        self.main_view.open_toolbar().click_button("backButton")

        self.ensure_app_has_quit()
        self.start_app()

        view = self.album_view.get_animated_album_view()
        self.assertThat(view.isOpen, Equals(False))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from lomiri_gallery_app.emulators.gallery_utils import GalleryUtils


class AlbumsView(GalleryUtils):

    def __init__(self, app):
        self.app = app

    def get_albums_tab_button(self):
        """Returns the photos tab."""
        return self.app.select_single("AbstractButton", buttonIndex=0)

    def number_of_albums_in_albums_view(self):
        """Returns the number of albums shown in albums overview"""
        albums = self.app.select_many("CheckerboardDelegate",
                                      objectName="checkerboardDelegate")
        return len(albums)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from lomiriuitoolkit import emulators as toolkit_emulators


class MediaViewer(toolkit_emulators.LomiriUIToolkitEmulatorBase):
    """A class that makes it easy to interact with the media viewer"""

    def get_edit_spinner(self):
        return self.select_single(
            "ActivityIndicator",
            objectName="busySpinner"
        )

    def ensure_spinner_not_running(self):
        """Wait for spinner to stop running"""
        spinner = self.get_edit_spinner()
        spinner.running.wait_for(False)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from lomiri_gallery_app.emulators.gallery_utils import GalleryUtils


class MediaSelector(GalleryUtils):
    """A class that makes it easy to interact with the gallery app"""

    def __init__(self, app):
        self.app = app

    def get_media_selector(self):
        """Return the media selector"""
        return self.app.wait_select_single("MediaSelector")

    def get_second_photo(self):
        """Return the second photo item"""
        selector = self.get_media_selector()
        # get the first row in the selector
        events = self.app.select_many('OrganicMediaList')
        photos = events[0].select_many( 'QQuickItem', objectName='eventPhoto')
        # return the item itself
        return photos[0]

    def ensure_fully_open(self):
        """Ensure media selector is fully open"""
        media_selector = self.get_media_selector()
        media_selector.opacity.wait_for(1.0)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

import logging
import re

from testtools.matchers import GreaterThan, LessThan, Equals, Is
from autopilot.matchers import Eventually

from autopilot.introspection.dbus import StateNotFoundError
from autopilot import logging as autopilot_logging

from lomiri_gallery_app.emulators.gallery_utils import(
    GalleryUtils,
    GalleryAppException,
)

logger = logging.getLogger(__name__)


class AlbumView(GalleryUtils):
    """An emulator class that makes it easy to interact with the gallery app"""

    def __init__(self, app):
        super(AlbumView, self).__init__(self)
        self.app = app

    def get_animated_album_view(self):
        """Returns the album view"""
        return self.app.wait_select_single("AlbumViewerAnimated",
                                           objectName="albumViewerAnimated")

    def get_album_view(self):
        """Returns the album view"""
        return self.app.wait_select_single("AlbumViewer",
                                           objectName="albumViewer")

    def get_first_photo(self):
        """Returns the first photo in a newly opened album"""
        return self.app.select_many(
            "FramePortrait", visible=True, isLoaded=True)[0]

    def get_album_photo_view(self):
        """Returns the photo view of the album viewer"""
        return self.app.select_single("PopupPhotoViewer")

    def get_spread_view(self):
        """Returns the inner spread view to access the pages"""
        view = self.get_album_view()
        return view.select_single(
            "AlbumSpreadViewer",
            objectName="spreadViewer"
        )

    def number_of_photos(self):
        """Returns the numer of visible photos"""
        return len(self.app.select_many("FramePortrait",
                                        visible=True, isLoaded=True))

    def media_selector_loader(self):
        """Returns the loader for the media selector"""
        return self.app.select_single("QQuickLoader",
                                      objectName="albumMediaSelectorLoader")

    def get_plus_icon_empty_album(self):
        """Returns the plus icon visible in empty albums"""
        return self.app.select_single("QQuickImage",
                                      objectName="albumCoverAddPhotosImage",
                                      visible=True)

    def ensure_animated_fully_closed(self):
        """Ensure the animated album view is fully closed"""
        animated_viewer = self.get_animated_album_view()
        animated_viewer.isOpen.wait_for(False)
        animated_viewer.animationRunning.wait_for(False)

    def ensure_album_view_fully_closed(self):
        """Ensure the album view is fully closed"""
        view = self.get_album_view()
        view.visible.wait_for(False)

    def ensure_media_selector_is_fully_closed(self):
        """Ensure media selector is fully closed"""
        loader = self.media_selector_loader()
        loader.status.wait_for(0)

    def _swipe_setup(self, page_number, direction):
        self.album = self.get_album_view()
        self.spread = self.get_spread_view()
        self.album.animationRunning.wait_for(False)

        x, y, w, h = self.spread.globalRect
        mid_y = y + h // 2
        mid_x = x + w // 2

        if 'left' == direction:
            matcher = LessThan
            self.pointing_device.drag(
                mid_x, mid_y,  # Start
                x + w, mid_y,  # Finish
                rate=1
            )

        elif 'right' == direction:
            matcher = GreaterThan
            self.pointing_device.drag(
                mid_x, mid_y,  # Start
                x, mid_y,      # Finish
                rate=1
            )
        else:
            raise GalleryAppException(
                'Direction can be "right" or "left" '
                'you choose "{}"'.format(direction))

        self.album.animationRunning.wait_for(False)
        self.spread.viewingPage.wait_for(matcher(page_number))

    def swipe_page_left(self, page_number):
        '''Swipe page to the left

        :param page_number: The starting page number you are swiping from
        '''
        self._swipe_setup(page_number, 'left')

    def swipe_page_right(self, page_number):
        '''Swipe page to the right

        :param page_number: The starting page number you are swiping from
        '''
        self._swipe_setup(page_number, 'right')

    def _get_remove_from_album_dialog(self):
        """Returns the photo viewer remove from album dialog."""
        return self.app.wait_select_single(
            "Dialog",
            objectName="removePhotoFromAlbumDialog")

    def _remove_from_album_dialog_shown(self):
        dialog = self.app.select_many("Dialog",
                                      objectName="removePhotoFromAlbumDialog")
        return len(dialog) >= 1

    def _get_remove_from_album_popover_remove_item(self):
        """Returns remove button of the remove from album popover."""
        return self.app.wait_select_single("Button",
                                           objectName="removeFromAlbumButton",
                                           visible=True)

    def _get_remove_from_album_popover_delete_item(self):
        """Returns delete button of the remove from album popover."""
        return self.app.wait_select_single(
            "Button",
            objectName="removeFromAlbumAndDeleteButton",
            visible=True)

    def _get_remove_from_album_popover_cancel_item(self):
        """Returns cancel button of the remove from album popover."""
        return self.app.wait_select_single(
            "Button",
            objectName="removeFromAlbumCancelButton",
            visible=True)

    def _ensure_remove_from_album_dialog_is_open(self):
        """Ensure that the remove from album dialog is fully opened."""
        self.assertThat(self._remove_from_album_dialog_shown,
                        Eventually(Is(True)))
        remove_dialog = self._get_remove_from_album_dialog()
        self.assertThat(remove_dialog.visible, Eventually(Equals(True)))
        self.assertThat(remove_dialog.opacity, Eventually(Equals(1)))

    def _ensure_remove_from_album_dialog_is_close(self):
        """Ensure that the remove from album dialog is fully closed."""
        self.assertThat(self._remove_from_album_dialog_shown,
                        Eventually(Is(False)))

    @autopilot_logging.log_action(logger.info)
    def click_remove_from_album_remove_button(self):
        """Click on the remove from album button of the remove dialog."""
        self._ensure_remove_from_album_dialog_is_open()

        remove_item = self._get_remove_from_album_popover_remove_item()
        self.pointing_device.click_object(remove_item)

        self._ensure_remove_from_album_dialog_is_close()

    @autopilot_logging.log_action(logger.info)
    def click_remove_from_album_delete_button(self):
        """Click on the remove and delete of the remove dialog."""
        self._ensure_remove_from_album_dialog_is_open()

        delete_item = self._get_remove_from_album_popover_delete_item()
        self.pointing_device.click_object(delete_item)

        self._ensure_remove_from_album_dialog_is_close()

    @autopilot_logging.log_action(logger.info)
    def click_remove_from_album_cancel_button(self):
        """Click on the cancel of the remove dialog."""
        self._ensure_remove_from_album_dialog_is_open()

        cancel_item = self._get_remove_from_album_popover_cancel_item()
        self.pointing_device.click_object(cancel_item)

        self._ensure_remove_from_album_dialog_is_close()

    @autopilot_logging.log_action(logger.info)
    def click_first_photo(self):
        """Click on the first photo of the album and return the path of it."""
        photo = self.get_first_photo()
        images = photo.select_many('QQuickImage')
        path = ''
        for i in images:
            if str(i.source).startswith('image://thumbnailer/file://'):
                path = re.sub('^image://thumbnailer/file://', '',
                              i.source).split('?')[0]
        self.pointing_device.click_object(photo)
        return path

    def _get_header(self):
        return self.app.select_single('PageHeader',
                                      objectName='albumViewerHeader')

    def _open_overflow(self):
        overflow_button = self._get_header().select_single(
            objectName='overflow_action_button')
        self.pointing_device.click_object(overflow_button)
        return self.get_root_instance().wait_select_single(
            objectName='actions_overflow_panel',
            visible=True)

    def click_action_button(self, action_object_name):
        header = self._get_header()
        header.visible.wait_for(True)

        try:
            object_name = action_object_name + "_button"
            button = header.select_single(objectName=object_name)
            self.pointing_device.click_object(button)
        except StateNotFoundError:
            object_name = action_object_name + "_button"
            popover = self._open_overflow()
            button = popover.select_single(objectName=object_name)
            self.pointing_device.click_object(button)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from lomiriuitoolkit import emulators as toolkit_emulators


class AlbumEditor(toolkit_emulators.LomiriUIToolkitEmulatorBase):
    """A class that makes it easy to interact with the album editor"""

    def close(self):
        cover_image = self.album_cover_image()
        # click left of the cover
        x, y, w, h = cover_image.globalRect
        self.pointing_device.move(x - 10, y + (h // 2))
        # workaround lp:1247698 (get rid of toolbar)
        self.pointing_device.click()
        self.pointing_device.click()
        self.ensure_fully_closed

    def ensure_fully_open(self):
        self.animationRunning.wait_for(False)
        self.active.wait_for(True)

    def ensure_fully_closed(self):
        self.active.wait_for(False)

    def album_editor(self):
        """Returns the album editor."""
        return self

    def album_title_entry_field(self):
        """Returns the album title input box."""
        editor = self.album_editor()
        return editor.wait_select_single("TextEditOnClick",
                                         objectName="albumTitleField")

    def click_title_field(self):
        self.pointing_device.click_object(self.album_title_entry_field())

    def album_subtitle_entry_field(self):
        """Returns the album subtitle input box."""
        editor = self.album_editor()
        return editor.wait_select_single("TextEditOnClick",
                                         objectName="albumSubtitleField")

    def click_subtitle_field(self):
        self.pointing_device.click_object(self.album_subtitle_entry_field())

    def album_cover_image(self):
        album_cover = self.wait_select_single("AlbumCover", visible=True)
        return album_cover.wait_select_single(
            "QQuickImage",
            objectName="albumCoverImage"
        )

    def add_photos(self):
        self.pointing_device.click_object(self._plus_icon())

    def _plus_icon(self):
        """Returns the plus icon to add photos"""
        return self.wait_select_single("QQuickImage",
                                       objectName="albumCoverAddPhotosImage",
                                       visible=True)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.


"""Tests for the Gallery App"""

from testtools.matchers import Equals, NotEquals, Is, GreaterThan
from autopilot.matchers import Eventually
from autopilot.platform import model
from autopilot.introspection.dbus import StateNotFoundError

from lomiri_gallery_app.tests import GalleryTestCase
from lomiri_gallery_app.emulators.events_view import EventsView

from os import environ as env
from os.path import exists
import shutil


class TestEventsView(GalleryTestCase):
    """Tests the main gallery features"""
    envDesktopMode = None

    @property
    def events_view(self):
        return EventsView(self.app)

    def setUp(self):
        self.ARGS = []

        self.envDesktopMode = env.get("DESKTOP_MODE")

        if model() == "Desktop":
            env["DESKTOP_MODE"] = "1"
        else:
            env["DESKTOP_MODE"] = "0"

        # This is needed to wait for the application to start.
        # In the testfarm, the application may take some time to show up.
        super(TestEventsView, self).setUp()
        self.main_view.switch_to_tab("eventsTab")
        """Wait for the data to be loaded and displayed"""
        self.assertThat(lambda: self.events_view.number_of_events(),
                        Eventually(GreaterThan(0)))

    def tearDown(self):
        if self.envDesktopMode:
            env["DESKTOP_MODE"] = self.envDesktopMode
        else:
            del env["DESKTOP_MODE"]

        super(TestEventsView, self).tearDown()

    def get_events_view(self):
        return self.app.wait_select_single("EventsOverview")

    def enable_select_mode(self):
        self.main_view.get_header().click_action_button("selectButton")

    def check_header_button_exist(self, button):
        buttonName = button + "_button"
        try:
            self.main_view.select_single(objectName=buttonName)
        except StateNotFoundError:
            return False
        return True

    def test_select_button_cancel(self):
        """Clicking the cancel button after clicking the select button must
           hide the toolbar automatically."""
        events_view = self.get_events_view()
        self.assertFalse(events_view.inSelectionMode)

        self.enable_select_mode()
        self.assertTrue(events_view.inSelectionMode)

        self.main_view.get_header().click_custom_back_button()

        self.assertFalse(events_view.inSelectionMode)

    def test_delete_a_photo(self):
        """Selecting a photo must make the delete button clickable."""
        self.assertThat(lambda: exists(self.sample_jpg_files[3]),
                        Eventually(Equals(True)))

        self.enable_select_mode()
        self.events_view.select_photo(self.sample_jpg_files[3])
        self.main_view.get_header().click_action_button("deleteButton")
        self.assertThat(self.gallery_utils.delete_dialog_shown,
                        Eventually(Is(True)))

        self.gallery_utils.click_delete_dialog_cancel_button()
        self.assertThat(self.gallery_utils.delete_dialog_shown,
                        Eventually(Is(False)))

        self.assertThat(lambda: exists(self.sample_jpg_files[3]),
                        Eventually(Equals(True)))

        self.main_view.get_header().click_action_button("deleteButton")
        self.assertThat(self.gallery_utils.delete_dialog_shown,
                        Eventually(Is(True)))

        self.gallery_utils.click_delete_dialog_delete_button()
        self.assertThat(self.gallery_utils.delete_dialog_shown,
                        Eventually(Is(False)))

        self.assertThat(lambda: exists(self.sample_jpg_files[3]),
                        Eventually(Equals(False)))

    def test_share_single_photo(self):
        """Selecting a photo must make the share button clickable."""
        self.assertThat(lambda: exists(self.sample_jpg_files[3]),
                        Eventually(Equals(True)))
        self.enable_select_mode()
        self.events_view.select_photo(self.sample_jpg_files[3])
        self.main_view.get_header().click_action_button("shareButton")
        share_picker = self.events_view.get_share_peer_picker()
        self.assertThat(share_picker.visible, Eventually(Equals(True)))
        self.main_view.get_header().click_back_button()
        self.assertThat(share_picker.visible, Eventually(Equals(False)))

    def test_share_multiple_photos(self):
        """Selecting multiple photos must make the share button clickable."""
        self.assertThat(lambda: exists(self.sample_jpg_files[2]),
                        Eventually(Equals(True)))
        self.assertThat(lambda: exists(self.sample_jpg_files[3]),
                        Eventually(Equals(True)))
        self.enable_select_mode()
        self.events_view.select_photo(self.sample_jpg_files[2], 1)
        self.events_view.select_photo(self.sample_jpg_files[3], 0)
        self.main_view.get_header().click_action_button("shareButton")
        share_picker = self.events_view.get_share_peer_picker()
        self.assertThat(share_picker.visible, Eventually(Equals(True)))
        self.main_view.get_header().click_back_button()
        self.assertThat(share_picker.visible, Eventually(Equals(False)))

    def test_adding_a_video(self):
        if model() == "Desktop":
            before = self.events_view.get_event(0)
            video_file = "video.mp4"
            shutil.copyfile(self.sample_dir+"/option01/"+video_file,
                            self.sample_destination_dir+"/"+video_file)
            video_file = "video.mkv"
            shutil.copyfile(self.sample_dir+"/option01/"+video_file,
                            self.sample_destination_dir+"/"+video_file)
            after = self.events_view.get_event(0)
            self.assertThat(lambda: str(after),
                            Eventually(NotEquals(str(before))))
            self.assertThat(
                lambda: self.events_view.number_of_photos_in_events(),
                Eventually(Equals(4)))

    # Check if Camera Button is not visible at Desktop mode
    def test_camera_button_visible(self):
        cameraButtonVisible = self.check_header_button_exist("cameraButton")
        if model() == "Desktop":
            self.assertThat(cameraButtonVisible, Equals(False))
        else:
            self.assertThat(cameraButtonVisible, Equals(True))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from lomiri_gallery_app.emulators.gallery_utils import(
    GalleryAppException,
    GalleryUtils
)


class EventsView(GalleryUtils):

    def __init__(self, app):
        super(EventsView, self).__init__(self)
        self.app = app

    def get_event(self, event_number=0):
        """Return an event in the event view based on index number

        :param event_number: the index number of the organicEventItem to get
        """
        return self.app.select_single(
            'OrganicMediaList',
            objectName='organicEventItem{}'.format(int(event_number))
        )

    def number_of_events(self):
        """Return the number of events in the model behind the event view"""
        return self.app.select_single('EventsOverview')._eventCount

    def number_of_photos_in_events(self):
        """Return the number of photos in events"""

        overview = self.app.select_single('EventsOverview')
        photo_delegates = overview.select_many(
            "QQuickItem",
            objectName="eventPhoto"
        )
        return len(photo_delegates)

    def number_of_photos_in_event(self, event):
        """Return the number of photo delgated in an event"""
        photo_delegates = event.select_many(objectName='eventPhoto')
        return len(photo_delegates)

    def get_share_peer_picker(self):
        """Returns the photo viewer share picker."""
        return self.app.wait_select_single(objectName="sharePickerEvents",
                                           visible=True)

    def _get_image_in_event_view(self, image_name):
        """Return the photo of the gallery based on image name.

        :param image_name: the name of the photo in the event to return"""
        events = self.app.select_many('OrganicMediaList')
        for event in events:
            photos = event.select_many(
                'QQuickItem',
                objectName='eventPhoto'
            )
            for photo in photos:
                images = photo.select_many('QQuickImage')
                for image in images:
                    image.status.wait_for(1)
                    src = image.source.split('?')[0]
                    if str(src).endswith(image_name):
                        return image
        raise GalleryAppException(
            'Photo with image name {} could not be found'.format(image_name))

    def _get_item_in_event_view(self, image_name, event_index_num=0):
        """Return the item of the gallery based on image name.

        :param image_name: the name of the photo in the event to return"""
        event = self.get_event(event_index_num)
        photos = event.select_many(
            'QQuickItem',
            objectName='eventPhoto'
        )
        for photo in photos:
            images = photo.select_many('QQuickImage')
            for image in images:
                src = image.source.split('?')[0]
                if str(src).endswith(image_name):
                    return photo
        raise GalleryAppException(
            'Photo with image name {} could not be found'.format(image_name))

    def click_photo(self, photo_name):
        """Click photo with name and event

        :param photo_name: name of file to click
        """
        photo = self._get_image_in_event_view(photo_name)
        self.pointing_device.click_object(photo)

    def select_photo(self, photo_name, event_index_num=0):
        """Select photo with name and event

        :param photo_name: name of file to click
        :param event_index_num: index of event to click
        """
        photo = self._get_item_in_event_view(photo_name, event_index_num)
        checkbox = photo.select_single(objectName="selectionCheckbox")
        self.pointing_device.click_object(checkbox)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.


"""Tests the Photo editor of the gallery app."""

from testtools.matchers import Equals, NotEquals, GreaterThan
from autopilot.matchers import Eventually
from testtools import skipIf
from autopilot.platform import model

from lomiri_gallery_app.emulators import photo_viewer
from lomiri_gallery_app.emulators.media_viewer import MediaViewer
from lomiri_gallery_app.emulators.events_view import EventsView
from lomiri_gallery_app.tests import GalleryTestCase

import os
import shutil
from time import sleep
import unittest

"""
Class for common functionality of the photo viewing and photo editing
"""


class TestMediaViewerBase(GalleryTestCase):
    @property
    def photo_viewer(self):
        return photo_viewer.PhotoViewer(self.app)

    @property
    def events_view(self):
        return EventsView(self.app)

    def setUp(self):
        self.ARGS = []
        super(TestMediaViewerBase, self).setUp()
        self.main_view.switch_to_tab("eventsTab")

    def open_first_photo(self):
        self.assertThat(
            lambda: self.events_view.number_of_photos_in_events(),
            Eventually(GreaterThan(0))
        )

        self.events_view.click_photo(self.sample_jpg_files[3])

        photo_viewer_loader = self.photo_viewer.get_main_photo_viewer_loader()
        self.assertThat(photo_viewer_loader.loaded, Eventually(Equals(True)))
        sleep(1)
        photo_viewer = self.photo_viewer.get_main_photo_viewer()
        self.assertThat(photo_viewer.visible, Eventually(Equals(True)))


class TestPhotoViewer(TestMediaViewerBase):
    def setUp(self):
        super(TestPhotoViewer, self).setUp()
        self.open_first_photo()

    @unittest.skip("Temporarily disable as it fails in some cases, "
                   "supposedly due to problems with the infrastructure")
    def test_save_state(self):
        """Quitting the app once a photo has been opened will return
        to that same photo on restart"""
        path = self.photo_viewer.get_photo_component().select_single(
            "QQuickImage").source

        self.ensure_app_has_quit()
        self.start_app()

        photo_viewer = self.photo_viewer.get_main_photo_viewer()
        self.assertThat(photo_viewer.visible, Eventually(Equals(True)))
        new_path = self.photo_viewer.get_photo_component().select_single(
            "QQuickImage").source

        self.assertThat(path, Equals(new_path))

    @unittest.skip("Temporarily disable as it fails in some cases, "
                   "supposedly due to problems with the infrastructure")
    def test_no_save_state_on_back(self):
        """Quitting the app once a photo has been opened and then closed
        will not reopen a photo on restart"""
        self.main_view.open_toolbar().click_button("backButton")

        self.ensure_app_has_quit()
        self.start_app()

        photo_viewer_loader = self.photo_viewer.get_main_photo_viewer_loader()
        self.assertThat(photo_viewer_loader.source, Equals(""))

    def test_nav_bar_back_button(self):
        """Clicking the back button must close the photo."""
        photo_viewer_popup = self.main_view.select_single(
            photo_viewer.PopupPhotoViewer)
        photo_viewer_popup.click_action_button("backButton")
        viewer = self.photo_viewer.get_main_photo_viewer()
        self.assertThat(viewer.visible, Eventually(Equals(False)))

    @skipIf(model() == 'Desktop', 'Share not supported on desktop')
    def test_share_button(self):
        """Clicking the share button must show the ContentPeerPicker."""
        photo_viewer_popup = self.main_view.select_single(
            photo_viewer.PopupPhotoViewer)
        photo_viewer_popup.click_action_button("shareButton")
        share_picker = self.photo_viewer.get_share_peer_picker()
        self.assertThat(share_picker.visible, Eventually(Equals(True)))
        self.main_view.get_header().click_back_button()
        self.assertThat(share_picker.visible, Eventually(Equals(False)))

    def test_delete_photo_must_remove_it_from_filesystem(self):
        photo_component = self.main_view.select_single('GalleryPhotoComponent')
        source = photo_component.source
        file_path = source[len('image://thumbnailer/file://'):]
        self.assertTrue(os.path.exists(file_path))

        photo_viewer_popup = self.main_view.select_single(
            photo_viewer.PopupPhotoViewer)
        photo_viewer_popup.delete_current_photo()

        self.assertFalse(os.path.exists(file_path))

    def test_photo_delete_works(self):
        """Clicking the trash button must show the delete dialog."""
        # XXX This test must be split into multiple QML tests.
        # --elopio - 2015-14-16
        photo_viewer_popup = self.main_view.select_single(
            photo_viewer.PopupPhotoViewer)
        photo_viewer_popup.delete_current_photo(confirm=False)

        self.assertThat(lambda: os.path.exists(self.sample_jpg_files[3]),
                        Eventually(Equals(True)))

        photo_viewer_popup.delete_current_photo()
        self.assertThat(lambda: os.path.exists(self.sample_jpg_files[3]),
                        Eventually(Equals(False)))

        # Delete all other pictures and make sure the photo viewer closes
        photo_viewer_popup.delete_current_photo()
        photo_viewer_popup.delete_current_photo()
        photo_viewer_popup.delete_current_photo()
        photo_viewer_popup.delete_current_photo()

        self.assertThat(photo_viewer_popup.visible, Eventually(Equals(False)))

    def test_nav_bar_album_picker_button(self):
        """Clicking the album picker must show the picker dialog."""
        photo_viewer_popup = self.main_view.select_single(
            photo_viewer.PopupPhotoViewer)
        photo_viewer_popup.click_action_button("addButton")
        album_picker = self.photo_viewer.get_popup_album_picker()
        self.assertThat(album_picker.visible, Eventually(Equals(True)))

    def test_double_click_zoom(self):
        """Double clicking an opened photo must zoom it."""
        opened_photo = self.photo_viewer.get_photo_component()

        self.pointing_device.move_to_object(opened_photo)
        self.pointing_device.click()
        self.pointing_device.click()

        self.assertThat(opened_photo.fullyZoomed, Eventually(Equals(True)))

        self.pointing_device.click()
        self.pointing_device.click()

        self.assertThat(opened_photo.fullyUnzoomed, Eventually(Equals(True)))

    def test_swipe_change_image(self):
        """Swiping left and right on a photo should move to another photo"""
        list = self.photo_viewer.get_photos_list()
        self.assertThat(list.currentIndex, Eventually(Equals(0)))

        # Slide left should move to the next image
        x, y, w, h = list.globalRect
        mid_y = y + h // 2
        mid_x = x + w // 2
        self.pointing_device.drag(mid_x, mid_y, x + 10, mid_y, rate=5)

        self.assertThat(list.moving, Eventually(Equals(False)))
        self.assertThat(list.currentIndex, Eventually(Equals(1)))

        # Slide right should get us back to the start
        self.pointing_device.drag(mid_x, mid_y, x + w - 10, mid_y, rate=5)

        self.assertThat(list.moving, Eventually(Equals(False)))
        self.assertThat(list.currentIndex, Eventually(Equals(0)))

        # Slide right again shouldn't go anywhere
        self.pointing_device.drag(mid_x, mid_y, x + w - 10, mid_y, rate=5)

        self.assertThat(list.moving, Eventually(Equals(False)))
        self.assertThat(list.currentIndex, Eventually(Equals(0)))


class TestVideoViewer(TestMediaViewerBase):
    def test_video_loads_thumbnail(self):
        num_events = self.events_view.number_of_events()

        video_file = "video.mp4"
        shutil.copyfile(self.sample_dir+"/option01/"+video_file,
                        self.sample_destination_dir+"/"+video_file)

        self.assertThat(
            lambda: self.events_view.number_of_events(),
            Eventually(Equals(num_events + 1))
        )

        self.events_view.click_photo(video_file)

        photo_viewer_loader = self.photo_viewer.get_main_photo_viewer_loader()
        self.assertThat(photo_viewer_loader.loaded, Eventually(Equals(True)))
        sleep(1)
        photo_viewer = self.photo_viewer.get_main_photo_viewer()
        self.assertThat(photo_viewer.visible, Eventually(Equals(True)))
        photo_component = self.photo_viewer.get_photo_component()
        self.assertThat(photo_component.imageReady, Eventually(Equals(True)))

        os.remove(self.sample_destination_dir+"/"+video_file)


class TestPhotoEditor(TestMediaViewerBase):

    def setUp(self):
        super(TestPhotoEditor, self).setUp()
        self.open_first_photo()
        self.click_edit_button()
        self.media_view = self.app.select_single(MediaViewer)

    def click_edit_button(self):
        photo_viewer_popup = self.main_view.select_single(
            photo_viewer.PopupPhotoViewer)
        photo_viewer_popup.click_action_button("editButton")
        photo_editor = self.photo_viewer.get_photo_editor()
        self.assertThat(photo_editor.visible, (Eventually(Equals(True))))
        self.assertThat(photo_editor.opacity, (Eventually(Equals(1))))

    def test_photo_editor_crop(self):
        """Cropping a photo must crop it."""
        old_file_size = os.path.getsize(self.sample_jpg_files[3])

        self.photo_viewer.click_crop_button()

        crop_box = self.photo_viewer.get_crop_interactor()
        item_width = crop_box.width
        item_height = crop_box.height

        self.assertThat(crop_box.opacity, Eventually(Equals(1)))

        crop_corner = self.photo_viewer.get_top_left_crop_corner()
        x, y, h, w = crop_corner.globalRect
        x = x + w // 2
        y = y + h // 2
        self.pointing_device.drag(x, y,
                                  x + item_width // 2, y + item_height // 2)

        # wait for animation being finished
        crop_overlay = self.photo_viewer.get_crop_overlay()
        self.assertThat(crop_overlay.interpolationFactor,
                        Eventually(Equals(1.0)))

        crop_button = self.photo_viewer.get_crop_overlays_crop_icon()
        self.click_item(crop_button)
        self.media_view.ensure_spinner_not_running()

        self.main_view.get_header().click_custom_back_button()
        photo_viewer = self.photo_viewer.get_main_photo_viewer()
        self.assertThat(photo_viewer.visible, Eventually(Equals(True)))

        new_file_size = os.path.getsize(self.sample_jpg_files[3])
        self.assertThat(old_file_size > new_file_size, Equals(True))

    def test_photo_editor_rotate(self):
        """Makes sure that the photo editor inside the photo viewer works using
           the rotate function"""
        opened_photo = self.photo_viewer.get_opened_photo()
        item_height = opened_photo.height

        def is_landscape():
            return opened_photo.paintedWidth > opened_photo.paintedHeight
        self.assertThat(is_landscape(), Equals(True))

        self.photo_viewer.click_rotate_button()
        self.media_view.ensure_spinner_not_running()
        self.main_view.get_header().click_custom_back_button()

        self.assertThat(opened_photo.paintedHeight,
                        Eventually(Equals(item_height)))
        self.assertThat(lambda: is_landscape(),
                        Eventually(Equals(False)))

        self.click_edit_button()
        self.photo_viewer.click_rotate_button()
        self.media_view.ensure_spinner_not_running()
        self.main_view.get_header().click_custom_back_button()

        self.click_edit_button()
        self.photo_viewer.click_revert_button()
        self.photo_viewer.click_confirm_revert_button()
        self.main_view.get_header().click_custom_back_button()

        self.assertThat(opened_photo.paintedHeight,
                        Eventually(NotEquals(item_height)))
        is_landscape = opened_photo.paintedWidth > opened_photo.paintedHeight
        self.assertThat(is_landscape, Equals(True))

    @unittest.skip("Temporarily disable as the enhance feature is not "
                   "implemented on the new editor")
    def test_photo_editor_redo_undo_revert_enhance_states(self):
        undo_item = self.photo_viewer.get_undo_menu_item()
        redo_item = self.photo_viewer.get_redo_menu_item()
        revert_item = self.photo_viewer.get_revert_menu_item()

        self.assertThat(undo_item.enabled, Eventually(Equals(False)))
        self.assertThat(redo_item.enabled, Eventually(Equals(False)))
        self.assertThat(revert_item.enabled, Eventually(Equals(False)))

        self.photo_viewer.click_rotate_item()
        self.media_view.ensure_spinner_not_running()

        self.click_edit_button()
        undo_item = self.photo_viewer.get_undo_menu_item()
        redo_item = self.photo_viewer.get_redo_menu_item()
        revert_item = self.photo_viewer.get_revert_menu_item()

        self.assertThat(undo_item.enabled, Eventually(Equals(True)))
        self.assertThat(redo_item.enabled, Eventually(Equals(False)))
        self.assertThat(revert_item.enabled, Eventually(Equals(True)))

        self.photo_viewer.click_undo_item()
        self.media_view.ensure_spinner_not_running()

        self.click_edit_button()
        undo_item = self.photo_viewer.get_undo_menu_item()
        redo_item = self.photo_viewer.get_redo_menu_item()
        revert_item = self.photo_viewer.get_revert_menu_item()

        self.assertThat(undo_item.enabled, Eventually(Equals(False)))
        self.assertThat(redo_item.enabled, Eventually(Equals(True)))
        self.assertThat(revert_item.enabled, Eventually(Equals(False)))

        self.photo_viewer.click_redo_item()
        self.media_view.ensure_spinner_not_running()

        self.click_edit_button()
        undo_item = self.photo_viewer.get_undo_menu_item()
        redo_item = self.photo_viewer.get_redo_menu_item()
        revert_item = self.photo_viewer.get_revert_menu_item()

        self.assertThat(undo_item.enabled, Eventually(Equals(True)))
        self.assertThat(redo_item.enabled, Eventually(Equals(False)))
        self.assertThat(revert_item.enabled, Eventually(Equals(True)))

        self.photo_viewer.click_revert_item()
        self.media_view.ensure_spinner_not_running()

        self.click_edit_button()
        undo_item = self.photo_viewer.get_undo_menu_item()
        redo_item = self.photo_viewer.get_redo_menu_item()
        revert_item = self.photo_viewer.get_revert_menu_item()

        self.assertThat(undo_item.enabled, Eventually(Equals(True)))
        self.assertThat(redo_item.enabled, Eventually(Equals(False)))
        self.assertThat(revert_item.enabled, Eventually(Equals(False)))

        self.photo_viewer.click_enhance_item()
        self.media_view.ensure_spinner_not_running()

        self.click_edit_button()

        revert_item = self.photo_viewer.get_revert_menu_item()
        self.assertThat(lambda: revert_item.enabled, Eventually(Equals(True)))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from lomiri_gallery_app.emulators.gallery_utils import GalleryUtils


class PhotosView(GalleryUtils):

    def __init__(self, app):
        self.app = app

    def get_photo_in_photos_view_by_index(self, index):
        """Returns the photo with index in the photos view."""
        return self.select_many_retry(
            "QQuickItem",
            objectName="allPotosGridPhoto")[index]

    def get_first_photo_in_photos_view(self):
        """Returns the very first photo in the photos view."""
        return self.get_photo_in_photos_view_by_index(0)

    def number_of_photos(self):
        """Returns the number of events"""
        photo_delegates = self.app.select_many("QQuickItem",
                                               objectName="allPotosGridPhoto")
        return len(photo_delegates)

    def get_share_peer_picker(self):
        """Returns the photo viewer share picker."""
        return self.app.wait_select_single(objectName="sharePickerPhotos",
                                           visible=True)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.


"""Tests the Photos view of the gallery app."""

from testtools.matchers import Equals, Is
from testtools import skipUnless
from autopilot.matchers import Eventually
from autopilot.platform import model
from autopilot.introspection.dbus import StateNotFoundError

from lomiri_gallery_app.tests import GalleryTestCase
from lomiri_gallery_app.emulators.photos_view import PhotosView

from time import sleep
from os import environ as env
import unittest


class TestPhotosView(GalleryTestCase):
    envDesktopMode = None

    @property
    def photos_view(self):
        return PhotosView(self.app)

    def setUp(self):
        self.ARGS = []
        self.envDesktopMode = env.get("DESKTOP_MODE")

        if model() == "Desktop":
            env["DESKTOP_MODE"] = "1"
        else:
            env["DESKTOP_MODE"] = "0"

        super(TestPhotosView, self).setUp()
        self.switch_to_photos_tab()

    def tearDown(self):
        if self.envDesktopMode:
            env["DESKTOP_MODE"] = self.envDesktopMode
        else:
            del env["DESKTOP_MODE"]

        super(TestPhotosView, self).tearDown()

    def switch_to_photos_tab(self):
        self.main_view.switch_to_tab("photosTab")
        self.ensure_tabs_dont_move()

    def click_first_photo(self):
        photo = self.photos_view.get_first_photo_in_photos_view()
        self.click_item(photo)

    def select_photo_by_index(self, index):
        photo = self.photos_view.get_photo_in_photos_view_by_index(index)
        checkbox = photo.select_single(objectName="selectionCheckbox")
        self.click_item(checkbox)

    def select_first_photo(self):
        self.select_photo_by_index(0)

    def check_header_button_exist(self, button):
        buttonName = button + "_button"
        try:
            self.main_view.select_single(objectName=buttonName)
        except StateNotFoundError:
            return False
        return True

    def test_open_photo(self):
        self.click_first_photo()
        sleep(5)
        photo_viewer = self.photos_view.get_main_photo_viewer()
        self.assertThat(photo_viewer.visible, Eventually(Equals(True)))

    def test_select_button_cancel(self):
        """Clicking the cancel button after clicking the select button must
        hide the toolbar automatically."""
        photos_overview = self.app.select_single("PhotosOverview")
        self.assertFalse(photos_overview.inSelectionMode)

        self.main_view.get_header().click_action_button("selectButton")
        self.assertTrue(photos_overview.inSelectionMode)

        self.main_view.get_header().click_custom_back_button()

        self.assertFalse(photos_overview.inSelectionMode)

        first_photo = self.photos_view.get_first_photo_in_photos_view()
        self.tap_item(first_photo)
        self.assertTrue(photos_overview.inSelectionMode)

    def test_delete_photo_dialog_appears(self):
        """Selecting a photo must make the delete button clickable."""
        self.main_view.get_header().click_action_button("selectButton")
        self.select_first_photo()
        self.main_view.get_header().click_action_button("deleteButton")

        self.assertThat(self.gallery_utils.delete_dialog_shown,
                        Eventually(Is(True)))

        cancel_item = self.photos_view.get_delete_dialog_cancel_button()
        self.click_item(cancel_item)
        self.assertThat(self.gallery_utils.delete_dialog_shown,
                        Eventually(Is(False)))

    def test_delete_a_photo(self):
        """Must be able to select a photo and use the dialog to delete it."""
        number_of_photos = self.photos_view.number_of_photos()
        self.main_view.get_header().click_action_button("selectButton")
        self.select_first_photo()
        self.main_view.get_header().click_action_button("deleteButton")

        self.assertThat(self.gallery_utils.delete_dialog_shown,
                        Eventually(Is(True)))

        delete_item = self.photos_view.get_delete_dialog_delete_button()
        self.click_item(delete_item)
        self.assertThat(
            self.gallery_utils.delete_dialog_shown,
            Eventually(Is(False))
        )

        self.assertThat(lambda: self.photos_view.number_of_photos(),
                        Eventually(Equals(number_of_photos - 1)))

    def test_share_single_photo(self):
        """Selecting a photo must make the share button clickable."""
        self.main_view.get_header().click_action_button("selectButton")
        self.select_first_photo()
        self.main_view.get_header().click_action_button("shareButton")
        share_picker = self.photos_view.get_share_peer_picker()
        self.assertThat(share_picker.visible, Eventually(Equals(True)))
        self.main_view.get_header().click_back_button()
        self.assertThat(share_picker.visible, Eventually(Equals(False)))

    def test_share_multiple_photos(self):
        """Selecting multiple photos must make the share button clickable."""
        self.main_view.get_header().click_action_button("selectButton")
        self.select_photo_by_index(0)
        self.select_photo_by_index(1)
        self.main_view.get_header().click_action_button("shareButton")
        share_picker = self.photos_view.get_share_peer_picker()
        self.assertThat(share_picker.visible, Eventually(Equals(True)))
        self.main_view.get_header().click_back_button()
        self.assertThat(share_picker.visible, Eventually(Equals(False)))

    @unittest.skip("Temporarily disable as it fails in some cases, "
                   "supposedly due to problems with the infrastructure")
    def test_save_state(self):
        self.switch_to_photos_tab()

        tabs = self.main_view.select_single("Tabs")
        tab = tabs.get_current_tab()
        self.assertThat(tab.objectName, Equals("photosTab"))
        index = tab.index

        self.ensure_app_has_quit()
        self.start_app()

        tabs = self.main_view.select_single("Tabs")
        tab = tabs.get_current_tab()
        self.assertThat(tabs.selectedTabIndex, Eventually(Equals(index)))
        self.assertThat(tab.objectName, Equals("photosTab"))

    @skipUnless(
        model() == 'Desktop',
        'Key based tests only make sense on Desktop'
    )
    def test_toggle_fullscreen(self):
        self.switch_to_photos_tab()
        view = self.main_view
        self.assertThat(view.fullScreen, Eventually(Equals(False)))
        self.keyboard.press_and_release('F11')
        self.assertThat(view.fullScreen, Eventually(Equals(True)))
        self.keyboard.press_and_release('F11')
        self.assertThat(view.fullScreen, Eventually(Equals(False)))
        self.keyboard.press_and_release('F11')
        self.assertThat(view.fullScreen, Eventually(Equals(True)))
        self.keyboard.press_and_release('Escape')
        self.assertThat(view.fullScreen, Eventually(Equals(False)))
        self.keyboard.press_and_release('Escape')
        self.assertThat(view.fullScreen, Eventually(Equals(False)))

    # Check if Camera Button is not visible at Desktop mode
    def test_camera_button_visible(self):
        cameraButtonVisible = self.check_header_button_exist("cameraButton")
        if model() == "Desktop":
            self.assertThat(cameraButtonVisible, Equals(False))
        else:
            self.assertThat(cameraButtonVisible, Equals(True))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from lomiriuitoolkit import emulators as toolkit_emulators
from lomiri_gallery_app.emulators.gallery_utils import GalleryAppException


class PickerScreen(toolkit_emulators.MainView):

    def pick_button(self):
        return self.select_single(objectName="pickButton_button")

    def get_photos_tab_button(self):
        """Returns the photos tab."""
        return self.select_single("AbstractButton", buttonIndex=3)

    def events_view(self):
        return self.select_single("OrganicView", objectName="eventSelectView")

    def first_media_in_events_view(self):
        events_view = self.events_view()
        return events_view.select_many("OrganicItemInteraction",
                                       objectName="eventsViewPhoto")[0]

    def grid_view(self):
        return self.select_single("MediaGrid")

    def first_media_in_grid_view(self):
        grid_view = self.grid_view()
        return grid_view.select_many("OrganicItemInteraction")[0]

    def go_to_photos(self):
        """
        Switch to the photos tab on picker screen
        Return the Page object representing the photos
        """
        self.switch_to_tab('photosTab')
        return self.select_single(Page, objectName='photosPage')

    def click_pick_button(self):
        """Click on the pick button"""
        self.pointing_device.click_object(self.pick_button())


class Page(PickerScreen):
    """Class to represent photos page view from picker screen"""

    def _get_named_photo_element(self, photo_name):
        """
        Return the ShapeItem container object for the named photo.
        This object can be clicked to enable the photo to be selected.
        """
        elements = self.grid_view().select_many('QQuickImage')
        for element in elements:
            element.status.wait_for(1)
            src = element.source.split('?')[0]
            if str(src).endswith(photo_name):
                return element.get_parent()
        raise GalleryAppException(
            'Photo with image name {} could not be found'.format(photo_name))

    def select_named_photo(self, photo_name):
        """Select the named photo from the picker view."""
        photo_element = self._get_named_photo_element(photo_name)
        checkbox = photo_element.select_single(objectName="selectionCheckbox")
        self.pointing_device.click_object(checkbox)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013, 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from autopilot.introspection.dbus import StateNotFoundError
import logging
from lomiriuitoolkit import emulators as toolkit_emulators

logger = logging.getLogger(__name__)


class Toolbar(toolkit_emulators.Toolbar):
    """An emulator class that makes it easy to interact with the tool bar"""

    def click_button(self, object_name):
        """Click a button of the toolbar.

        The toolbar should be opened before clicking the button, or an
        exception will be raised. If the toolbar is closed for some reason
        (e.g., timer finishes) after moving the mouse cursor and before
        clicking the button, it is re-opened automatically by this function.

        Overriden from the toolkit because of bug http://pad.lv/1302706
        TODO remove this method once that bug is fixed on the toolkit.
        --elopio - 2014-04-04

        :parameter object_name: The QML objectName property of the button.
        :raise ToolkitEmulatorException: If there is no button with that object
            name.

        """
        # ensure the toolbar is open
        if not self.opened:
            raise toolkit_emulators.ToolkitEmulatorException(
                'Toolbar must be opened before calling click_button().')
        try:
            button = self._get_button(object_name)
        except StateNotFoundError:
            raise toolkit_emulators.ToolkitEmulatorException(
                'Button with objectName "{0}" not found.'.format(object_name))
        self.pointing_device.move_to_object(button)
        # ensure the toolbar is still open (may have closed due to timeout)
        self.open()
        # click the button
        self.pointing_device.click_object(button)

    def click_custom_button(self, object_name):
        try:
            button = self.select_single('Button', objectName=object_name)
            self.pointing_device.click_object(button)
        except StateNotFoundError:
            logger.error(
                'Button with objectName "{0}" not found.'.format(object_name)
            )
            raise



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""gallery autopilot tests."""

import os
import logging
import shutil
import signal

from testtools.matchers import Equals, NotEquals, GreaterThan
from autopilot.matchers import Eventually
from autopilot.platform import model
from autopilot.testcase import AutopilotTestCase
from autopilot.introspection import get_proxy_object_for_existing_process
from pkg_resources import resource_filename

from lomiriuitoolkit import emulators as toolkit_emulators
from lomiri_gallery_app.emulators import main_screen
from lomiri_gallery_app.emulators.gallery_utils import GalleryUtils

from time import sleep

logger = logging.getLogger(__name__)


class EnvironmentTypes:
    installed = "installed"
    local = "local"
    click = "click"


class GalleryTestCase(AutopilotTestCase):

    """A common test case class that provides several useful methods for
       gallery tests."""

    sample_file_source = "/sample01.jpg"
    tap_press_time = 1
    local_location = "../../src/gallery-app"

    _db = '~/.local/share/gallery.ubports/' \
          'database/gallery.sqlite'
    _thumbs = '~/.cache/gallery.ubports/thumbnails'

    _default_sample_destination_dir = "/tmp/gallery-ap_sd"

    ARGS = []

    @property
    def gallery_utils(self):
        return GalleryUtils(self.app)

    @property
    def main_view(self):
        return self.app.select_single(main_screen.MainScreen)

    def _get_environment_launch_type(self):
        """Returns the type of the environment in which to setup and launch
        gallery-app.

        """
        # Lets assume we are installed system wide if this file is somewhere
        # in /usr
        if os.path.realpath(__file__).startswith("/usr/"):
            return EnvironmentTypes.installed
        if model() == 'Desktop':
            return EnvironmentTypes.installed
        else:
            if os.path.exists(self.local_location):
                return EnvironmentTypes.local
            else:
                return EnvironmentTypes.click

    def _get_sample_destination_dir(self, env_type):
        if env_type == EnvironmentTypes.click:
            pic_dir = os.path.expanduser("~/Pictures")
            pic_bak_dir = pic_dir + '.apbak'
            # Only save and restore if it previously existed
            if os.path.exists(pic_dir):
                shutil.move(pic_dir, pic_bak_dir)
                self.addCleanup(
                    logger.debug, "Restoring backed up pics to %s" % pic_dir)
                self.addCleanup(shutil.move, pic_bak_dir, pic_dir)
            return pic_dir
        else:
            return self._default_sample_destination_dir

    def configure_db(self):
        db = os.path.expanduser(self._db)
        db_bak = db + '.apbak'
        # Only save and restore if it previously existed
        if os.path.exists(db):
            shutil.move(db, db_bak)
            self.addCleanup(shutil.move, db_bak, db)
        if not os.path.exists(os.path.dirname(db)):
            os.makedirs(os.path.dirname(db))
        mock_db = os.path.join(self.sample_destination_dir, '.database',
                               'gallery_confined.sqlite')
        shutil.move(mock_db, db)

    def configure_thumbnails(self):
        thumbs = os.path.expanduser(self._thumbs)
        thumbs_bak = thumbs + '.apbak'
        # Only save and restore if it previously existed
        if os.path.exists(thumbs):
            shutil.move(thumbs, thumbs_bak)
            self.addCleanup(shutil.move, thumbs_bak, thumbs)
        if not os.path.exists(os.path.dirname(thumbs)):
            os.makedirs(os.path.dirname(thumbs))
        mock_thumbs = os.path.join(self.sample_destination_dir, '.thumbnails')
        shutil.move(mock_thumbs, thumbs)

    def configure_sample_files(self, env_type):
        self.sample_dir = resource_filename('lomiri_gallery_app', 'data')
        self.sample_destination_dir = \
            self._get_sample_destination_dir(env_type)
        if (os.path.exists(self.sample_destination_dir)):
            shutil.rmtree(self.sample_destination_dir)
        self.assertFalse(os.path.exists(self.sample_destination_dir))

        default_data_dir = os.path.join(
            self.sample_dir,
            "default")

        self.sample_jpg_files = []
        for f in os.listdir(default_data_dir):
            if f.endswith(".jpg"):
                self.sample_jpg_files.append(os.path.join(
                    self.sample_destination_dir, f))

        shutil.copytree(default_data_dir, self.sample_destination_dir)
        for sample in self.sample_jpg_files:
            self.assertTrue(os.path.isfile(sample))

        self.sample_file_source = \
            default_data_dir + self.sample_file_source

        if env_type == EnvironmentTypes.click:
            self.configure_db()
            self.configure_thumbnails()

    def do_reset_config(self):
        config = os.path.expanduser(
            os.path.join("~", ".config", "lomiri-gallery-app.conf"))
        if os.path.exists(config):
            os.remove(config)

    def setUp(self):
        self.pointing_device = toolkit_emulators.get_pointing_device()
        super(GalleryTestCase, self).setUp()

        env_type = self._get_environment_launch_type()
        self.configure_sample_files(env_type)

        self.launch_gallery_app(env_type)

        self.addCleanup(shutil.rmtree, self.sample_destination_dir)
        self.addCleanup(logger.debug,
                        "Deleting %s" % self.sample_destination_dir)

        """ This is needed to wait for the application to start.
        In the testfarm, the application may take some time to show up."""
        qml_view = self.gallery_utils.get_qml_view()
        self.assertThat(qml_view.visible, Eventually(Equals(True)))
        """FIXME somehow on the server gallery sometimes is not fully started
        for switching to the albums view. Therefore this hack of sleeping"""
        sleep(2)

    def launch_gallery_app(self, env_type):
        if env_type == EnvironmentTypes.installed:
            self.launch_test_installed()
        elif env_type == EnvironmentTypes.local:
            self.launch_test_local()
        elif env_type == EnvironmentTypes.click:
            self.launch_test_click()
        else:
            raise ValueError("Unknown environment type: %s", env_type)

    def launch_test_local(self):
        logger.debug("Launching local gallery-app binary.")
        self.ARGS.append(self.sample_destination_dir)
        self.app = self.launch_test_application(
            self.local_location,
            *self.ARGS,
            emulator_base=toolkit_emulators.LomiriUIToolkitEmulatorBase)

    def launch_test_installed(self):
        if model() == 'Desktop':
            logger.debug(
                "Launching installed gallery-app binary for desktop."
            )
            self.ARGS.append(self.sample_destination_dir)
            self.app = self.launch_test_application(
                "gallery-app",
                *self.ARGS,
                emulator_base=toolkit_emulators.LomiriUIToolkitEmulatorBase)
        else:
            logger.debug(
                "Launching installed gallery-app binary for device."
            )
            self.ARGS.append("--desktop_file_hint="
                             "/usr/share/applications/gallery-app.desktop")
            self.ARGS.append(self.sample_destination_dir)
            self.app = self.launch_test_application(
                "gallery-app",
                *self.ARGS,
                app_type='qt',
                emulator_base=toolkit_emulators.LomiriUIToolkitEmulatorBase)

    def launch_test_click(self):
        '''
        Since this test runs under confinement, the only location photos
        are searchable in is ~/Pictures.
        '''
        logger.debug("Launching gallery-app via click package.")
        self.app = self.launch_click_package(
            package_id="gallery.ubports",
            app_uris=' '.join(self.ARGS),
            emulator_base=toolkit_emulators.LomiriUIToolkitEmulatorBase)

    def ui_update(self):
        """ Gives the program the time to update the UI"""
        sleep(0.1)

    def click_item(self, item, delay=0.1):
        """Does a mouse click on the passed item, and moved the mouse there
           before"""
        # In jenkins test may fail because we don't wait before clicking the
        # target so we add a little delay before click.
        if model() == 'Desktop' and delay <= 0.25:
            delay = 0.25

        self.pointing_device.move_to_object(item)
        sleep(delay)
        self.pointing_device.click()

    def tap_item(self, item):
        """Does a long mouse press on the passed item, and moved the mouse
           there before"""
        self.pointing_device.move_to_object(item)
        self.pointing_device.press()
        sleep(1)
        self.pointing_device.release()

    def switch_to_albums_tab(self):
        self.main_view.switch_to_tab("albumsTab")

        albums_loader = self.gallery_utils.get_albums_viewer_loader()
        self.assertThat(albums_loader.progress, Eventually(Equals(1)))

        # The next check assumes that at least one album is available
        # Check if the albums are availabe - they need some time to load.
        self.assertThat(lambda: len(self.gallery_utils.get_all_albums()),
                        Eventually(GreaterThan(0)))
        self.ensure_tabs_dont_move()

    def ensure_tabs_dont_move(self):
        # FIXME find a (functional) way to test if the tabs still move
        sleep(1)

    def open_album_at(self, position):
        album = self.album_view.get_album_at(position)
        self.click_item(album)
        self.ensure_view_is_fully_open()

    def open_first_album(self):
        self.open_album_at(0)

    def ensure_view_is_fully_open(self):
        view = self.album_view.get_album_view()
        self.assertThat(view.visible, Eventually(Equals(True)))

    def ensure_app_has_quit(self):
        """Terminate as gracefully as possible the application and ensure
        that it has fully quit before returning"""

        if model() == "Desktop":
            # On desktop to cleanly quit an app we just do the
            # equivalent of clicking on the close button in the window.
            self.keyboard.press_and_release("Alt+F4")
        else:
            # On unity8 at the moment we have no clean way to close the app.
            # So we ask the shell first to show the home, unfocusing our app,
            # which will save its state. Then we simply send it a SIGTERM to
            # force it to quit.
            # See bug https://bugs.launchpad.net/unity8/+bug/1261720 for more
            # details.
            from unity8 import process_helpers
            pid = process_helpers._get_unity_pid()
            unity8 = get_proxy_object_for_existing_process(pid=pid)
            shell = unity8.select_single("Shell")
            shell.slots.showHome()
            self.assertThat(shell.focusedApplicationId,
                            Eventually(NotEquals("gallery-app")))
            self.app.process.send_signal(signal.SIGTERM)

        # Either way, we wait for the underlying process to be fully finished.
        self.app.process.wait()
        self.assertIsNotNone(self.app.process.returncode)

    def get_delete_dialog(self):
        """Raises StateNotFoundError if get_delete_dialog fails."""
        delete_dialog = self.gallery_utils.get_delete_dialog()
        self.assertThat(delete_dialog.visible, Eventually(Equals(True)))
        self.assertThat(delete_dialog.opacity, Eventually(Equals(1)))
        return delete_dialog



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013, 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

import lomiriuitoolkit


class MainScreen(lomiriuitoolkit.MainView):
    """An emulator class that makes it easy to interact with the gallery app"""

    def get_header(self):
        """Return the Header emulator of the MainView."""
        return self.select_single(AppHeader, objectName='MainView_Header')


class AppHeader(lomiriuitoolkit.Header):
    """Header Autopilot helper.

    We override this helper because on the gallery the gesture to show the
    header when it's hidden it's not the default.

    """

    # XXX We are overriding internal methods that may change on the toolkit.
    # The helper on the toolkit needs a public method that will be safe to
    # override. Reported as bug http://pad.lv/1363591 --elopio - 2014-31-08

    def _is_visible(self):
        return self.visible

    def _show(self):
        self.pointing_device.click_object(self._get_top_container())
        self.visible.wait_for(True)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.


"""Tests the albums view of the gallery app."""

from testtools.matchers import Equals
from autopilot.matchers import Eventually
from autopilot.platform import model
from autopilot.introspection.dbus import StateNotFoundError

from lomiri_gallery_app.tests import GalleryTestCase
from lomiri_gallery_app.emulators.album_view import AlbumView
from lomiri_gallery_app.emulators.albums_view import AlbumsView
from lomiri_gallery_app.emulators.media_selector import MediaSelector
from lomiri_gallery_app.emulators import album_editor

from os import environ as env


class TestAlbumsView(GalleryTestCase):
    envDesktopMode = None

    @property
    def album_view(self):
        return AlbumView(self.app)

    @property
    def albums_view(self):
        return AlbumsView(self.app)

    @property
    def media_selector(self):
        return MediaSelector(self.app)

    def setUp(self):
        self.ARGS = []

        self.envDesktopMode = env.get("DESKTOP_MODE")

        if model() == "Desktop":
            env["DESKTOP_MODE"] = "1"
        else:
            env["DESKTOP_MODE"] = "0"

        super(TestAlbumsView, self).setUp()
        self.switch_to_albums_tab()

    def tearDown(self):
        if self.envDesktopMode:
            env["DESKTOP_MODE"] = self.envDesktopMode
        else:
            del env["DESKTOP_MODE"]

        super(TestAlbumsView, self).tearDown()

    def check_header_button_exist(self, button):
        buttonName = button + "_button"
        try:
            self.main_view.select_single(objectName=buttonName)
        except StateNotFoundError:
            return False
        return True

    def test_add_album(self):
        """Add one album, and checks if the number of albums went up by one"""
        albums = self.albums_view.number_of_albums_in_albums_view()
        self.main_view.get_header().click_action_button("addButton")
        # a photo must be added to the album for it to be saved
        plus = self.album_view.get_plus_icon_empty_album()
        self.click_item(plus)
        self.media_selector.ensure_fully_open()
        photo = self.media_selector.get_second_photo()
        checkbox = photo.select_single(objectName="selectionCheckbox")
        self.click_item(checkbox)
        self.main_view.get_header().click_action_button("addButton")

        self.assertThat(
            lambda: self.albums_view.number_of_albums_in_albums_view(),
            Eventually(Equals(albums+1))
        )

    def test_add_album_and_cancel(self):
        """Add one album, cancel it and checks if the number of albums does not
        change. Do this in the two different ways we have to cancel editing.
        """

        # Cancel by pressing back in the header
        albums = self.albums_view.number_of_albums_in_albums_view()
        self.main_view.get_header().click_action_button("addButton")
        editor = self.app.select_single(album_editor.AlbumEditor)
        editor.ensure_fully_open()
        self.main_view.get_header().click_custom_back_button()
        self.assertThat(
            lambda: self.albums_view.number_of_albums_in_albums_view(),
            Eventually(Equals(albums))
        )

        # Cancel by clicking outside of the album cover
        self.main_view.get_header().click_action_button("addButton")
        editor = self.app.select_single(album_editor.AlbumEditor)
        editor.ensure_fully_open()
        editor.close()

        self.assertThat(
            lambda: self.albums_view.number_of_albums_in_albums_view(),
            Eventually(Equals(albums))
        )

    # Check if Camera Button is not visible at Desktop mode
    def test_camera_button_visible(self):
        cameraButtonVisible = self.check_header_button_exist("cameraButton")
        if model() == "Desktop":
            self.assertThat(cameraButtonVisible, Equals(False))
        else:
            self.assertThat(cameraButtonVisible, Equals(True))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.


"""Tests the album editor of the gallery app"""

from testtools.matchers import Equals
from autopilot.matchers import Eventually

from lomiri_gallery_app.emulators.album_view import AlbumView
from lomiri_gallery_app.emulators.media_selector import MediaSelector
from lomiri_gallery_app.emulators import album_editor
from lomiri_gallery_app.tests import GalleryTestCase

from time import sleep


class TestAlbumEditor(GalleryTestCase):
    """Tests the album editor of the gallery app"""

    @property
    def album_view(self):
        return AlbumView(self.app)

    @property
    def media_selector(self):
        return MediaSelector(self.app)

    def setUp(self):
        self.ARGS = []
        super(TestAlbumEditor, self).setUp()
        self.switch_to_albums_tab()
        self.edit_first_album()

    def edit_first_album(self):
        first_album = self.gallery_utils.get_first_album()
        self.tap_item(first_album)
        edit_button = self.gallery_utils.get_edit_album_button()
        self.click_item(edit_button)
        editor = self.app.wait_select_single(album_editor.AlbumEditor)
        editor.ensure_fully_open()

    def test_album_title_fields(self):
        """tests the title and sub title"""
        editor = self.app.select_single(album_editor.AlbumEditor)
        title_field = editor.album_title_entry_field()
        subtitle_field = editor.album_subtitle_entry_field()

        text = "My Photo Album"
        self.assertThat(title_field.text, Eventually(Equals(text)))

        text = "Ubuntu"
        self.assertThat(subtitle_field.text, Eventually(Equals(text)))

        editor.click_title_field()
        self.assertThat(title_field.activeFocus, Eventually(Equals(True)))
        self.keyboard.press_and_release("Ctrl+a")
        text = "Photos"
        self.keyboard.type(text)
        # due to some reason the album title is not updated unless it loses the
        # focus. So we click on the subtitle field.
        editor.click_subtitle_field()
        self.assertThat(subtitle_field.activeFocus, Eventually(Equals(True)))
        self.assertThat(title_field.text, Eventually(Equals(text)))

        self.keyboard.press_and_release("Ctrl+a")
        text = "U1"
        self.keyboard.type(text)
        editor.click_title_field()
        self.assertThat(subtitle_field.text, Eventually(Equals(text)))

    def test_add_photo(self):
        """Tests adding a photo using the media selector"""
        # first open, but cancel before adding a photo
        editor = self.app.select_single(album_editor.AlbumEditor)
        editor.add_photos()
        self.media_selector.ensure_fully_open()

        sleep(5)
        self.main_view.get_header().click_custom_back_button()

        editor.ensure_fully_closed()

        sleep(5)
        self.open_first_album()
        num_photos_start = self.album_view.number_of_photos()
        self.assertThat(num_photos_start, Equals(1))

        # should click away of any photo to toggle header
        photo = self.album_view.get_first_photo()
        x, y, w, h = photo.globalRect
        self.pointing_device.move(x + 40, y + h + 40)
        self.pointing_device.click()

        self.album_view.click_action_button("backButton")
        self.album_view.ensure_album_view_fully_closed()

        # now open to add a photo
        self.edit_first_album()
        editor = self.app.select_single(album_editor.AlbumEditor)
        editor.add_photos()
        self.media_selector.ensure_fully_open()

        photo = self.media_selector.get_second_photo()
        checkbox = photo.select_single(objectName="selectionCheckbox")
        self.click_item(checkbox)
        self.main_view.get_header().click_action_button("addButton")
        editor = self.app.select_single(album_editor.AlbumEditor)
        editor.ensure_fully_closed()

        self.album_view.ensure_album_view_fully_closed()
        self.open_first_album()
        num_photos = self.album_view.number_of_photos()
        self.assertThat(num_photos, Equals(num_photos_start + 1))

    def test_cover_image(self):
        """Test to change the album cover image"""
        editor = self.app.select_single(album_editor.AlbumEditor)
        cover_image = editor.album_cover_image()
        self.assertThat(
            cover_image.source.endswith("album-cover-default-large.png"),
            Equals(True))

        # click somewhere rather at the bottom of the cover
        x, y, w, h = cover_image.globalRect
        self.pointing_device.move(x + int(w / 2), y + h - int(h / 10))
        self.pointing_device.click()

        green_item = self.gallery_utils.get_cover_menu_item("green")
        self.click_item(green_item)

        self.assertThat(lambda: cover_image.source.endswith(
            "album-cover-green-large.png"), Eventually(Equals(True)))



#!/usr/bin/python3

import os
import shutil
import sqlite3
import sys
from pathlib import Path

organization = "gallery.ubports"
application = "gallery.ubports"
old_application = "com.ubuntu.gallery"
camera_application = "camera.ubports"
old_camera_application = "com.ubuntu.camera"

xdg_config_home = Path(os.environ.get("XDG_CONFIG_HOME",
                                      Path.home() / ".config"))
xdg_data_home = Path(
    os.environ.get("XDG_DATA_HOME", Path.home() / ".local" / "share")
)
old_config_file = xdg_config_home / organization / f"{old_application}.conf"
new_config_file = xdg_config_home / organization / f"{application}.conf"
if old_config_file.is_file() and not new_config_file.exists():
    old_config_file.rename(new_config_file)

old_pictures_path = Path.home() / "Pictures" / old_camera_application
pictures_path = Path.home() / "Pictures" / camera_application
if old_pictures_path.is_dir() and not pictures_path.exists():
    old_pictures_path.rename(pictures_path)

db_file = xdg_data_home / organization / "database" / "gallery.sqlite"
if db_file.is_file():
    con = sqlite3.connect(db_file)
    with con:
        con.execute(
            f"UPDATE MediaTable SET filename = replace(filename, "
            f"'/{old_camera_application}/', '/{camera_application}/')",
        )
    con.close()

if len(sys.argv) > 1:
    os.execvp(sys.argv[1], sys.argv[1:])



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
import lomiriuitoolkit.emulators

from autopilot.testcase import AutopilotTestCase

from time import sleep


class GalleryAppException(Exception):
    pass


class GalleryUtils(AutopilotTestCase):
    """An emulator class that makes it easy to interact with
       general components of the gallery app."""

    retry_delay = 0.2

    def __init__(self, app):
        self.app = app
        self.pointing_device = lomiriuitoolkit.emulators.get_pointing_device()

    def select_many_retry(self, object_type, **kwargs):
        """Returns the item that is searched for with app.select_many
        In case of no item was not found (not created yet) a second attempt is
        taken 1 second later"""
        items = self.app.select_many(object_type, **kwargs)
        tries = 10
        while len(items) < 1 and tries > 0:
            sleep(self.retry_delay)
            items = self.app.select_many(object_type, **kwargs)
            tries -= 1
        return items

    def get_qml_view(self):
        """Get the main QML view"""
        return self.app.select_single("QQuickView")

    def get_main_photo_viewer_loader(self):
        """Returns the loader item for the PhotoViewer."""
        return self.app.select_single("QQuickLoader",
                                      objectName="photoViewerLoader")

    def get_main_photo_viewer(self):
        """Returns the MediaListView."""
        return self.app.wait_select_single("MediaListView",
                                           objectName="mediaListView")

    def get_albums_viewer_loader(self):
        """Returns the loader item for the AlbumsOverview."""
        return self.app.select_single("QQuickLoader",
                                      objectName="albumsCheckerboardLoader")

    def get_delete_dialog(self):
        """Returns the delete dialog in the events view."""
        return self.app.wait_select_single("DeleteDialog",
                                           objectName="deleteDialog")

    def delete_dialog_shown(self):
        dialog = self.app.select_many(
            "DeleteDialog",
            objectName="deleteDialog")
        return len(dialog) >= 1

    def get_delete_dialog_delete_button(self):
        """Returns the delete button of the delete popover."""
        return self.app.wait_select_single("Button",
                                           objectName="deleteDialogYes",
                                           visible=True)

    def get_delete_dialog_cancel_button(self):
        """Returns the cancel button of the delete popover."""
        return self.app.wait_select_single("Button",
                                           objectName="deleteDialogNo",
                                           visible=True)

    def get_all_albums(self):
        """Returns all albums in the albums view"""
        albums = self.select_many_retry("CheckerboardDelegate",
                                        objectName="checkerboardDelegate")
        return albums

    def get_first_album(self):
        """Returns the first album in the albums view."""
        return self.get_album_at(0)

    def get_album_at(self, position):
        """Returns the albums at this position in the albums view"""
        albums = self.select_many_retry("CheckerboardDelegate",
                                        objectName="checkerboardDelegate")
        albums = sorted(
            albums, key=lambda album: (album.globalRect.y, album.globalRect.x))
        return albums[position]

    def get_edit_album_button(self):
        """Returns the edit album button in the album popover"""
        return self.app.select_single("Standard",
                                      objectName="editAlbumListItem")

    def get_cover_menu_item(self, name):
        """Returns the item of the cover menu with the name *name*

        :raises StateNotFoundError: if there is no item with the name *name*

        """
        return self.app.select_single(
            "Standard",
            objectName="albumCoverMenuItem_" + name
        )

    def _ensure_delete_dialog_visible(self):
        delete_dialog = self.get_delete_dialog()
        delete_dialog.opacity.wait_for(1)

    def _click_dialog_button(self, button):
        self._ensure_delete_dialog_visible()
        self.pointing_device.click_object(button)

    def click_delete_dialog_cancel_button(self):
        button = self.get_delete_dialog_cancel_button()
        self._click_dialog_button(button)

    def click_delete_dialog_delete_button(self):
        button = self.get_delete_dialog_delete_button()
        self._click_dialog_button(button)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

import logging

from autopilot.introspection.dbus import StateNotFoundError
import autopilot.logging
import lomiriuitoolkit

from lomiri_gallery_app.emulators import main_screen
from lomiri_gallery_app.emulators.gallery_utils import(
    GalleryAppException,
    GalleryUtils
)


logger = logging.getLogger(__name__)


class PopupPhotoViewer(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):
    def _get_header(self):
        main = self.get_root_instance().select_single(main_screen.MainScreen)
        return main.select_single('PageHeader',
                                  objectName='photoViewerHeader')

    def _open_overflow(self):
        overflow_button = self._get_header().select_single(
            objectName='overflow_action_button')
        self.pointing_device.click_object(overflow_button)
        return self.get_root_instance().wait_select_single(
            objectName='actions_overflow_panel',
            visible=True)

    def click_action_button(self, action_object_name):
        header = self._get_header()
        if not header.visible:
            main = self.get_root_instance().select_single(
                main_screen.MainScreen)
            x, y, w, h = main.globalRect
            self.pointing_device.move(x + (w // 2), y + (h // 2))
            self.pointing_device.click()
            header.visible.wait_for(True)

        try:
            object_name = action_object_name + "_button"
            button = header.select_single(objectName=object_name)
            self.pointing_device.click_object(button)
        except StateNotFoundError:
            object_name = action_object_name + "_button"
            popover = self._open_overflow()
            button = popover.select_single(objectName=object_name)
            self.pointing_device.click_object(button)

    @autopilot.logging.log_action(logger.info)
    def delete_current_photo(self, confirm=True):
        self.click_action_button("deleteButton")
        if confirm:
            self.confirm_delete_photo()
        else:
            self.cancel_delete_photo()

    @autopilot.logging.log_action(logger.debug)
    def confirm_delete_photo(self):
        self._click_delete_dialog_button("Yes")

    def _click_delete_dialog_button(self, name):
        delete_dialog = self._get_delete_dialog()
        button = delete_dialog.wait_select_single(
            "Button", objectName="deletePhotoDialog" + name, visible=True)
        self.pointing_device.click_object(button)
        delete_dialog.wait_until_destroyed()

    def _get_delete_dialog(self):
        delete_dialog = self.get_root_instance().wait_select_single(
            objectName="deletePhotoDialog")
        delete_dialog.visible.wait_for(True)
        delete_dialog.opacity.wait_for(1)
        return delete_dialog

    @autopilot.logging.log_action(logger.debug)
    def cancel_delete_photo(self):
        self._click_delete_dialog_button('No')


class PhotoViewer(GalleryUtils):

    def __init__(self, app):
        super(PhotoViewer, self).__init__(self)
        self.app = app

    def get_popup_album_picker(self):
        """Returns the photo viewer album pickers."""
        return self.app.wait_select_single("PopupAlbumPicker",
                                           objectName="popupAlbumPicker")

    def get_share_peer_picker(self):
        """Returns the photo viewer share picker."""
        return self.app.wait_select_single(objectName="sharePicker",
                                           visible=True)

    def get_photo_editor(self):
        """Returns the photo edit dialog."""
        return self.app.wait_select_single("PhotoEditor")

    def get_revert_to_original_dialog(self):
        """Returns the revert to original dialog."""
        return self.app.wait_select_single("Dialog",
                                           objectName="revertPromptDialog")

    def get_cancel_revert_to_original_button(self):
        """Returns the revert to original cancel button."""
        return self.get_revert_to_original_dialog().wait_select_single(
            "Button",
            objectName="cancelRevertButton",
            visible=True)

    def get_confirm_revert_to_original_button(self):
        """Returns the revert to original confirm button."""
        return self.get_revert_to_original_dialog().wait_select_single(
            "Button",
            objectName="confirmRevertButton",
            visible=True)

    def get_photo_component(self):
        # Was using a list index (lp:1247711). Still needs fixing, I'm not
        # convinced this is a suitable way to select the correct item.
        return self.app.wait_select_single(
            "SingleMediaViewer",
            objectName="openedMedia0"
        )

    def get_photos_list(self):
        return self.app.wait_select_single("MediaListView")

    def get_editor_actions_bar(self):
        """Returns the actions bar for the editor."""
        return self.app.select_single("ActionsBar",
                                      objectName="editorActionsBar")

    def get_editor_action_button_by_text(self, button_text):
        """Returns the action button from the editor by text."""
        actions_bar = self.get_editor_actions_bar()
        buttons = actions_bar.select_many('AbstractButton')
        for button in buttons:
            if str(button.text) == button_text:
                return button
        raise GalleryAppException(
            'Editor action button {} could not be found'.format(button_text))

    def get_crop_action_button(self):
        """Returns the crop item of the edit dialog."""
        return self.get_editor_action_button_by_text("Crop")

    def get_rotate_action_button(self):
        """Returns the rotate item of the edit dialog."""
        return self.get_editor_action_button_by_text("Rotate")

    def get_undo_menu_item(self):
        """Returns the undo item of the edit dialog."""
        return self.app.select_single("Standard", objectName="undoListItem")

    def get_redo_menu_item(self):
        """Returns the redo item of the edit dialog."""
        return self.app.select_single("Standard", objectName="redoListItem")

    def get_revert_action_button(self):
        """Returns the revert to original menu item in the edit dialog."""
        return self.get_editor_action_button_by_text("Revert to Original")

    def get_auto_enhance_menu_item(self):
        """Returns the 'auto enhance' menu item in the edit dialog."""
        return self.app.select_single("Standard", objectName='enhanceListItem')

    def get_delete_popover_cancel_item(self):
        """Returns the cancel button of the delete popover."""
        return self.app.wait_select_single("Button",
                                           objectName="deletePhotoDialogNo",
                                           visible=True)

    def get_opened_photo(self):
        """Returns the first opened photo."""
        return self.app.wait_select_single("SingleMediaViewer",
                                           objectName="openedMedia0")

    def get_crop_interactor(self):
        """Returns the crop interactor."""
        return self.app.wait_select_single("CropInteractor",
                                           objectName="cropInteractor")

    def get_crop_overlay(self):
        """Returns the crop overlay."""
        return self.app.wait_select_single("CropOverlay",
                                           objectName="cropOverlay")

    def get_top_left_crop_corner(self):
        """Returns the top left corner of the crop overlay for dragging."""
        return self.app.wait_select_single("CropCorner",
                                           objectName="topLeftCropCorner")

    def get_crop_overlays_crop_icon(self):
        """Returns the crop icon of the crop overlay."""
        return self.app.wait_select_single("Button",
                                           objectName="centerCropIcon",
                                           visible=True)

    def get_edit_preview(self):
        """Returns the edit preview."""
        return self.app.wait_select_single("EditPreview",
                                           objectName="editPreview")

    def _click_item(self, item):
        self.pointing_device.click_object(item)

    def click_rotate_button(self):
        rotate_item = self.get_rotate_action_button()
        self._click_item(rotate_item)

    def click_crop_button(self):
        crop_item = self.get_crop_action_button()
        self._click_item(crop_item)

    def click_undo_item(self):
        undo_item = self.get_undo_menu_item()
        self._click_item(undo_item)

    def click_redo_item(self):
        redo_item = self.get_redo_menu_item()
        self._click_item(redo_item)

    def click_revert_button(self):
        revert_item = self.get_revert_action_button()
        self._click_item(revert_item)

    def click_cancel_revert_button(self):
        cancel_item = self.get_cancel_revert_to_original_button()
        self._click_item(cancel_item)

    def click_confirm_revert_button(self):
        confirm_item = self.get_confirm_revert_to_original_button()
        self._click_item(confirm_item)

    def click_enhance_item(self):
        enhance_item = self.get_auto_enhance_menu_item()
        self._click_item(enhance_item)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.


"""Tests the Photos view of the gallery app."""

from testtools.matchers import Equals
from autopilot.matchers import Eventually

from lomiri_gallery_app.emulators.picker_screen import PickerScreen
from lomiri_gallery_app.tests import GalleryTestCase
import unittest


class TestPickerMode(GalleryTestCase):

    @property
    def picker_view(self):
        return self.app.wait_select_single(PickerScreen)

    def setUp(self):
        self.ARGS = ['--pick-mode']
        super(TestPickerMode, self).setUp()

    def select_first_event_media(self):
        first_media = self.picker_view.first_media_in_events_view()
        checkbox = first_media.select_single(objectName="selectionCheckbox")
        self.click_item(checkbox)

    def select_first_grid_media(self):
        first_media = self.picker_view.first_media_in_events_view()
        self.click_item(first_media)

    def test_pick_first_photo(self):
        """Check if the button enabled state follows the selection"""
        pick_button = self.picker_view.pick_button()
        self.assertThat(pick_button.enabled, Eventually(Equals(False)))
        first_events_media = self.picker_view.first_media_in_events_view()
        self.assertThat(
            first_events_media.isSelected,
            Eventually(Equals(False))
        )

        self.select_first_event_media()

        pick_button = self.picker_view.pick_button()
        self.assertThat(pick_button.enabled, Eventually(Equals(True)))
        self.assertThat(
            first_events_media.isSelected,
            Eventually(Equals(True))
        )

        self.select_first_event_media()

        pick_button = self.picker_view.pick_button()
        self.assertThat(pick_button.enabled, Eventually(Equals(False)))
        self.assertThat(
            first_events_media.isSelected,
            Eventually(Equals(False))
        )

    def test_pick_named_photo(self):
        """Select a named photo and press Pick button."""
        photos_page = self.picker_view.go_to_photos()
        pick_button = self.picker_view.pick_button()
        self.assertFalse(pick_button.enabled)

        # create the image location path based on sample location
        image_file = 'sample02.jpg'
        photos_page.select_named_photo(image_file)
        self.assertThat(pick_button.enabled, Eventually(Equals(True)))
        self.picker_view.click_pick_button()

    def test_selection_synchronisation(self):
        """Checks if the selection is the same for both views"""
        first_events_media = self.picker_view.first_media_in_events_view()
        self.assertThat(
            first_events_media.isSelected,
            Eventually(Equals(False))
        )

        self.select_first_event_media()
        self.assertThat(
            first_events_media.isSelected,
            Eventually(Equals(True))
        )

        self.picker_view.switch_to_next_tab()

        first_grid_media = self.picker_view.first_media_in_grid_view()
        self.assertThat(first_grid_media.isSelected, Eventually(Equals(True)))

    @unittest.skip("Temporarily disable as it fails in some cases, "
                   "supposedly due to problems with the infrastructure")
    def test_save_picker_state(self):
        self.picker_view.switch_to_tab("photosTab")
        self.ensure_tabs_dont_move()

        tabs = self.picker_view.select_single("Tabs")
        tab = tabs.get_current_tab()
        self.assertThat(tab.objectName, Equals("photosTab"))
        index = tab.index

        self.ensure_app_has_quit()
        self.start_app()

        tabs = self.picker_view.select_single("Tabs")
        tab = tabs.get_current_tab()
        self.assertThat(tabs.selectedTabIndex, Eventually(Equals(index)))
        self.assertThat(tab.objectName, Equals("photosTab"))
